import _extends from "@babel/runtime/helpers/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/objectWithoutPropertiesLoose";
var _excluded = ["id", "children", "className", "style", "triggerEvent", "data", "propsFromTrigger", "keyMatcher", "onClick", "disabled", "hidden", "closeOnClick", "handlerEvent"];
import React, { useRef } from 'react';
import cx from 'classnames';
import { useItemTrackerContext } from './ItemTrackerProvider';
import { NOOP, CssClass } from '../constants';
import { getPredicateValue, isFn } from './utils';
import { contextMenu } from '../core';
export var Item = function Item(_ref) {
  var _extends2, _cx;
  var id = _ref.id,
    children = _ref.children,
    className = _ref.className,
    style = _ref.style,
    triggerEvent = _ref.triggerEvent,
    data = _ref.data,
    propsFromTrigger = _ref.propsFromTrigger,
    keyMatcher = _ref.keyMatcher,
    _ref$onClick = _ref.onClick,
    onClick = _ref$onClick === void 0 ? NOOP : _ref$onClick,
    _ref$disabled = _ref.disabled,
    disabled = _ref$disabled === void 0 ? false : _ref$disabled,
    _ref$hidden = _ref.hidden,
    hidden = _ref$hidden === void 0 ? false : _ref$hidden,
    _ref$closeOnClick = _ref.closeOnClick,
    closeOnClick = _ref$closeOnClick === void 0 ? true : _ref$closeOnClick,
    _ref$handlerEvent = _ref.handlerEvent,
    handlerEvent = _ref$handlerEvent === void 0 ? 'onClick' : _ref$handlerEvent,
    rest = _objectWithoutPropertiesLoose(_ref, _excluded);
  var itemNode = useRef();
  var itemTracker = useItemTrackerContext();
  var handlerParams = {
    id: id,
    data: data,
    triggerEvent: triggerEvent,
    props: propsFromTrigger
  };
  var isDisabled = getPredicateValue(disabled, handlerParams);
  var isHidden = getPredicateValue(hidden, handlerParams);
  function handleClick(e) {
    handlerParams.event = e;
    e.stopPropagation();
    if (!isDisabled) {
      !closeOnClick ? onClick(handlerParams) : dispatchUserHanlder();
    }
  }

  // provide a feedback to the user that the item has been clicked before closing the menu
  function dispatchUserHanlder() {
    var node = itemNode.current;
    node.focus();
    node.addEventListener('animationend',
    // defer, required for react 17
    function () {
      return setTimeout(contextMenu.hideAll);
    }, {
      once: true
    });
    node.classList.add(CssClass.itemClickedFeedback);
    onClick(handlerParams);
  }
  function registerItem(node) {
    if (node && !isDisabled) {
      itemNode.current = node;
      itemTracker.set(node, {
        node: node,
        isSubmenu: false,
        keyMatcher: !isDisabled && isFn(keyMatcher) && function (e) {
          if (keyMatcher(e)) {
            e.stopPropagation();
            e.preventDefault();
            handlerParams.event = e;
            dispatchUserHanlder();
          }
        }
      });
    }
  }
  function handleKeyDown(e) {
    if (e.key === 'Enter' || e.key === ' ') {
      e.stopPropagation();
      handlerParams.event = e;
      dispatchUserHanlder();
    }
  }
  if (isHidden) return null;
  return /*#__PURE__*/React.createElement("div", _extends({}, _extends({}, rest, (_extends2 = {}, _extends2[handlerEvent] = handleClick, _extends2)), {
    className: cx(CssClass.item, className, (_cx = {}, _cx["" + CssClass.itemDisabled] = isDisabled, _cx)),
    style: style,
    onKeyDown: handleKeyDown,
    ref: registerItem,
    tabIndex: -1,
    role: "menuitem",
    "aria-disabled": isDisabled
  }), /*#__PURE__*/React.createElement("div", {
    className: CssClass.itemContent
  }, children));
};