import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/objectWithoutPropertiesLoose";
import _extends from "@babel/runtime/helpers/extends";
var _excluded = ["id", "theme", "style", "className", "animation", "preventDefaultOnKeydown", "disableBoundariesCheck", "onVisibilityChange"];
function _createForOfIteratorHelperLoose(r, e) { var t = "undefined" != typeof Symbol && r[Symbol.iterator] || r["@@iterator"]; if (t) return (t = t.call(r)).next.bind(t); if (Array.isArray(r) || (t = _unsupportedIterableToArray(r)) || e && r && "number" == typeof r.length) { t && (r = t); var o = 0; return function () { return o >= r.length ? { done: !0 } : { done: !1, value: r[o++] }; }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }
function _unsupportedIterableToArray(r, a) { if (r) { if ("string" == typeof r) return _arrayLikeToArray(r, a); var t = {}.toString.call(r).slice(8, -1); return "Object" === t && r.constructor && (t = r.constructor.name), "Map" === t || "Set" === t ? Array.from(r) : "Arguments" === t || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(t) ? _arrayLikeToArray(r, a) : void 0; } }
function _arrayLikeToArray(r, a) { (null == a || a > r.length) && (a = r.length); for (var e = 0, n = Array(a); e < a; e++) n[e] = r[e]; return n; }
import React, { useEffect, useReducer, useRef, useState } from 'react';
import cx from 'classnames';
import { ItemTrackerProvider } from './ItemTrackerProvider';
import { eventManager } from '../core/eventManager';
import { useItemTracker } from '../hooks';
import { createKeyboardController } from './keyboardController';
import { CssClass, EVENT, hideOnEvents } from '../constants';
import { cloneItems, getMousePosition, isFn, isStr } from './utils';
import { flushSync } from 'react-dom';
function reducer(state, payload) {
  return _extends({}, state, isFn(payload) ? payload(state) : payload);
}
export var Menu = function Menu(_ref) {
  var _cx3;
  var id = _ref.id,
    theme = _ref.theme,
    style = _ref.style,
    className = _ref.className,
    _ref$animation = _ref.animation,
    animation = _ref$animation === void 0 ? 'fade' : _ref$animation,
    _ref$preventDefaultOn = _ref.preventDefaultOnKeydown,
    preventDefaultOnKeydown = _ref$preventDefaultOn === void 0 ? true : _ref$preventDefaultOn,
    _ref$disableBoundarie = _ref.disableBoundariesCheck,
    disableBoundariesCheck = _ref$disableBoundarie === void 0 ? false : _ref$disableBoundarie,
    onVisibilityChange = _ref.onVisibilityChange,
    rest = _objectWithoutPropertiesLoose(_ref, _excluded);
  var _useReducer = useReducer(reducer, {
      x: 0,
      y: 0,
      visible: false,
      triggerEvent: {},
      propsFromTrigger: null,
      willLeave: false
    }),
    state = _useReducer[0],
    setState = _useReducer[1];
  var nodeRef = useRef(null);
  var itemTracker = useItemTracker();
  var _useState = useState(function () {
      return createKeyboardController();
    }),
    menuController = _useState[0];
  var wasVisible = useRef();
  var visibilityId = useRef();

  // subscribe event manager
  useEffect(function () {
    eventManager.on(id, show).on(EVENT.HIDE_ALL, hide);
    return function () {
      eventManager.off(id, show).off(EVENT.HIDE_ALL, hide);
    };
    // hide rely on setState(dispatch), which is guaranted to be the same across render
  }, [id, animation, disableBoundariesCheck]);

  // collect menu items for keyboard navigation
  useEffect(function () {
    !state.visible ? itemTracker.clear() : menuController.init(itemTracker);
  }, [state.visible, menuController, itemTracker]);
  function checkBoundaries(x, y) {
    if (nodeRef.current && !disableBoundariesCheck) {
      var _window = window,
        innerWidth = _window.innerWidth,
        innerHeight = _window.innerHeight;
      var _nodeRef$current = nodeRef.current,
        offsetWidth = _nodeRef$current.offsetWidth,
        offsetHeight = _nodeRef$current.offsetHeight;
      if (x + offsetWidth > innerWidth) x -= x + offsetWidth - innerWidth;
      if (y + offsetHeight > innerHeight) y -= y + offsetHeight - innerHeight;
    }
    return {
      x: x,
      y: y
    };
  }

  // when the menu is transitioning from not visible to visible,
  // the nodeRef is attached to the dom element this let us check the boundaries
  useEffect(function () {
    // state.visible and state{x,y} are updated together
    if (state.visible) setState(checkBoundaries(state.x, state.y));
  }, [state.visible]);

  // subscribe dom events
  useEffect(function () {
    function preventDefault(e) {
      if (preventDefaultOnKeydown) e.preventDefault();
    }
    function handleKeyboard(e) {
      switch (e.key) {
        case 'Enter':
        case ' ':
          if (!menuController.openSubmenu()) hide();
          break;
        case 'Escape':
          hide();
          break;
        case 'ArrowUp':
          preventDefault(e);
          menuController.moveUp();
          break;
        case 'ArrowDown':
          preventDefault(e);
          menuController.moveDown();
          break;
        case 'ArrowRight':
          preventDefault(e);
          menuController.openSubmenu();
          break;
        case 'ArrowLeft':
          preventDefault(e);
          menuController.closeSubmenu();
          break;
        default:
          menuController.matchKeys(e);
          break;
      }
    }
    if (state.visible) {
      window.addEventListener('keydown', handleKeyboard);
      for (var _iterator = _createForOfIteratorHelperLoose(hideOnEvents), _step; !(_step = _iterator()).done;) {
        var ev = _step.value;
        window.addEventListener(ev, hide);
      }
    }
    return function () {
      window.removeEventListener('keydown', handleKeyboard);
      for (var _iterator2 = _createForOfIteratorHelperLoose(hideOnEvents), _step2; !(_step2 = _iterator2()).done;) {
        var _ev = _step2.value;
        window.removeEventListener(_ev, hide);
      }
    };
  }, [state.visible, menuController, preventDefaultOnKeydown]);
  function show(_ref2) {
    var event = _ref2.event,
      props = _ref2.props,
      position = _ref2.position,
      children = _ref2.children;
    event.stopPropagation();
    var p = position || getMousePosition(event);
    // check boundaries when the menu is already visible
    var _checkBoundaries = checkBoundaries(p.x, p.y),
      x = _checkBoundaries.x,
      y = _checkBoundaries.y;
    flushSync(function () {
      setState({
        visible: true,
        willLeave: false,
        x: x,
        y: y,
        triggerEvent: event,
        propsFromTrigger: props,
        children: children
      });
    });
    window.clearTimeout(visibilityId.current);
    if (!wasVisible.current && isFn(onVisibilityChange)) {
      onVisibilityChange(true);
      wasVisible.current = true;
    }
  }
  function hide(e) {
    if (e != null && (
    // Safari trigger a click event when you ctrl + trackpad
    e.button === 2 || e.ctrlKey) &&
    // Firefox trigger a click event when right click occur
    e.type !== 'contextmenu') return;
    animation && (isStr(animation) || 'exit' in animation && animation.exit) ? setState(function (state) {
      return {
        willLeave: state.visible
      };
    }) : setState(function (state) {
      return {
        visible: state.visible ? false : state.visible
      };
    });
    visibilityId.current = window.setTimeout(function () {
      isFn(onVisibilityChange) && onVisibilityChange(false);
      wasVisible.current = false;
    });
  }
  function handleAnimationEnd() {
    if (state.willLeave && state.visible) {
      flushSync(function () {
        return setState({
          visible: false,
          willLeave: false
        });
      });
    }
  }
  function computeAnimationClasses() {
    if (isStr(animation)) {
      var _cx;
      return cx((_cx = {}, _cx["" + CssClass.animationWillEnter + animation] = visible && !willLeave, _cx["" + CssClass.animationWillLeave + animation + " " + CssClass.animationWillLeave + "'disabled'"] = visible && willLeave, _cx));
    } else if (animation && 'enter' in animation && 'exit' in animation) {
      var _cx2;
      return cx((_cx2 = {}, _cx2["" + CssClass.animationWillEnter + animation.enter] = animation.enter && visible && !willLeave, _cx2["" + CssClass.animationWillLeave + animation.exit + " " + CssClass.animationWillLeave + "'disabled'"] = animation.exit && visible && willLeave, _cx2));
    }
    return null;
  }
  var visible = state.visible,
    triggerEvent = state.triggerEvent,
    propsFromTrigger = state.propsFromTrigger,
    x = state.x,
    y = state.y,
    willLeave = state.willLeave,
    children = state.children;
  var cssClasses = cx(CssClass.menu, className, (_cx3 = {}, _cx3["" + CssClass.theme + theme] = theme, _cx3), computeAnimationClasses());

  // TODO: switch to translate instead of top left
  // requires an additional dom element around the menu
  return /*#__PURE__*/React.createElement(ItemTrackerProvider, {
    value: itemTracker
  }, visible && /*#__PURE__*/React.createElement("div", _extends({}, rest, {
    className: cssClasses,
    onAnimationEnd: handleAnimationEnd,
    style: _extends({}, style, {
      left: x,
      top: y,
      opacity: 1
    }),
    ref: nodeRef,
    role: "menu"
  }), cloneItems(children, {
    propsFromTrigger: propsFromTrigger,
    triggerEvent: triggerEvent
  })));
};