import _extends from "@babel/runtime/helpers/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/objectWithoutPropertiesLoose";
var _excluded = ["arrow", "children", "disabled", "hidden", "label", "className", "triggerEvent", "propsFromTrigger", "style"];
import React, { useRef } from 'react';
import cx from 'classnames';
import { ItemTrackerProvider, useItemTrackerContext } from './ItemTrackerProvider';
import { useItemTracker } from '../hooks';
import { CssClass } from '../constants';
import { cloneItems, getPredicateValue } from './utils';
import { Arrow } from './Arrow';
import { RightSlot } from './RightSlot';
export var Submenu = function Submenu(_ref) {
  var _cx;
  var arrow = _ref.arrow,
    children = _ref.children,
    _ref$disabled = _ref.disabled,
    disabled = _ref$disabled === void 0 ? false : _ref$disabled,
    _ref$hidden = _ref.hidden,
    hidden = _ref$hidden === void 0 ? false : _ref$hidden,
    label = _ref.label,
    className = _ref.className,
    triggerEvent = _ref.triggerEvent,
    propsFromTrigger = _ref.propsFromTrigger,
    style = _ref.style,
    rest = _objectWithoutPropertiesLoose(_ref, _excluded);
  var parentItemTracker = useItemTrackerContext();
  var itemTracker = useItemTracker();
  var submenuNode = useRef(null);
  var handlerParams = {
    triggerEvent: triggerEvent,
    props: propsFromTrigger
  };
  var isDisabled = getPredicateValue(disabled, handlerParams);
  var isHidden = getPredicateValue(hidden, handlerParams);
  function setPosition() {
    var node = submenuNode.current;
    if (node) {
      var bottom = CssClass.submenu + "-bottom";
      var right = CssClass.submenu + "-right";

      // reset to default position before computing position
      node.classList.remove(bottom, right);
      var rect = node.getBoundingClientRect();
      if (rect.right > window.innerWidth) node.classList.add(right);
      if (rect.bottom > window.innerHeight) node.classList.add(bottom);
    }
  }
  function trackRef(node) {
    if (node && !isDisabled) parentItemTracker.set(node, {
      node: node,
      isSubmenu: true,
      submenuRefTracker: itemTracker,
      setSubmenuPosition: setPosition
    });
  }
  if (isHidden) return null;
  var cssClasses = cx(CssClass.item, className, (_cx = {}, _cx["" + CssClass.itemDisabled] = isDisabled, _cx));
  return /*#__PURE__*/React.createElement(ItemTrackerProvider, {
    value: itemTracker
  }, /*#__PURE__*/React.createElement("div", _extends({}, rest, {
    className: cssClasses,
    ref: trackRef,
    tabIndex: -1,
    role: "menuitem",
    "aria-haspopup": true,
    "aria-disabled": isDisabled,
    onMouseEnter: setPosition,
    onTouchStart: setPosition
  }), /*#__PURE__*/React.createElement("div", {
    className: CssClass.itemContent,
    onClick: function onClick(e) {
      return e.stopPropagation();
    }
  }, label, /*#__PURE__*/React.createElement(RightSlot, null, arrow || /*#__PURE__*/React.createElement(Arrow, null))), /*#__PURE__*/React.createElement("div", {
    className: CssClass.menu + " " + CssClass.submenu,
    ref: submenuNode,
    style: style
  }, cloneItems(children, {
    propsFromTrigger: propsFromTrigger,
    // @ts-ignore: injected by the parent
    triggerEvent: triggerEvent
  }))));
};