import { IPublicModelPluginContext } from '@gant-lowcode/lowcode-types';
/**
 * 高阶组件（HOC）：为组件提供 view 插件上下文。
 *
 * @param {React.ComponentType} Component - 需要被封装的组件。
 * @param {string|string[]} viewName - 视图名称或视图名称数组，用于过滤特定的视图插件上下文。
 * @returns {React.ComponentType} 返回封装后的组件。
 *
 * @example
 * // 用法示例（函数组件）:
 * const EnhancedComponent = ProvideViewPluginContext(MyComponent, "viewName");
 */
export declare const ProvideViewPluginContext: (Component: any, viewName?: string | string[]) => (props: {
    [key: string]: any;
    pluginContext?: IPublicModelPluginContext;
}) => JSX.Element;
