"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
exports.__esModule = true;
exports.Menu = void 0;
var _objectWithoutPropertiesLoose2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutPropertiesLoose"));
var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));
var _react = _interopRequireWildcard(require("react"));
var _classnames = _interopRequireDefault(require("classnames"));
var _ItemTrackerProvider = require("./ItemTrackerProvider");
var _eventManager = require("../core/eventManager");
var _hooks = require("../hooks");
var _keyboardController = require("./keyboardController");
var _constants = require("../constants");
var _utils = require("./utils");
var _reactDom = require("react-dom");
var _excluded = ["id", "theme", "style", "className", "animation", "preventDefaultOnKeydown", "disableBoundariesCheck", "onVisibilityChange"];
function _getRequireWildcardCache(e) { if ("function" != typeof WeakMap) return null; var r = new WeakMap(), t = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(e) { return e ? t : r; })(e); }
function _interopRequireWildcard(e, r) { if (!r && e && e.__esModule) return e; if (null === e || "object" != typeof e && "function" != typeof e) return { "default": e }; var t = _getRequireWildcardCache(r); if (t && t.has(e)) return t.get(e); var n = { __proto__: null }, a = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) { var i = a ? Object.getOwnPropertyDescriptor(e, u) : null; i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u]; } return n["default"] = e, t && t.set(e, n), n; }
function _createForOfIteratorHelperLoose(r, e) { var t = "undefined" != typeof Symbol && r[Symbol.iterator] || r["@@iterator"]; if (t) return (t = t.call(r)).next.bind(t); if (Array.isArray(r) || (t = _unsupportedIterableToArray(r)) || e && r && "number" == typeof r.length) { t && (r = t); var o = 0; return function () { return o >= r.length ? { done: !0 } : { done: !1, value: r[o++] }; }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }
function _unsupportedIterableToArray(r, a) { if (r) { if ("string" == typeof r) return _arrayLikeToArray(r, a); var t = {}.toString.call(r).slice(8, -1); return "Object" === t && r.constructor && (t = r.constructor.name), "Map" === t || "Set" === t ? Array.from(r) : "Arguments" === t || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(t) ? _arrayLikeToArray(r, a) : void 0; } }
function _arrayLikeToArray(r, a) { (null == a || a > r.length) && (a = r.length); for (var e = 0, n = Array(a); e < a; e++) n[e] = r[e]; return n; }
function reducer(state, payload) {
  return (0, _extends2["default"])({}, state, (0, _utils.isFn)(payload) ? payload(state) : payload);
}
var Menu = exports.Menu = function Menu(_ref) {
  var _cx3;
  var id = _ref.id,
    theme = _ref.theme,
    style = _ref.style,
    className = _ref.className,
    _ref$animation = _ref.animation,
    animation = _ref$animation === void 0 ? 'fade' : _ref$animation,
    _ref$preventDefaultOn = _ref.preventDefaultOnKeydown,
    preventDefaultOnKeydown = _ref$preventDefaultOn === void 0 ? true : _ref$preventDefaultOn,
    _ref$disableBoundarie = _ref.disableBoundariesCheck,
    disableBoundariesCheck = _ref$disableBoundarie === void 0 ? false : _ref$disableBoundarie,
    onVisibilityChange = _ref.onVisibilityChange,
    rest = (0, _objectWithoutPropertiesLoose2["default"])(_ref, _excluded);
  var _useReducer = (0, _react.useReducer)(reducer, {
      x: 0,
      y: 0,
      visible: false,
      triggerEvent: {},
      propsFromTrigger: null,
      willLeave: false
    }),
    state = _useReducer[0],
    setState = _useReducer[1];
  var nodeRef = (0, _react.useRef)(null);
  var itemTracker = (0, _hooks.useItemTracker)();
  var _useState = (0, _react.useState)(function () {
      return (0, _keyboardController.createKeyboardController)();
    }),
    menuController = _useState[0];
  var wasVisible = (0, _react.useRef)();
  var visibilityId = (0, _react.useRef)();

  // subscribe event manager
  (0, _react.useEffect)(function () {
    _eventManager.eventManager.on(id, show).on(_constants.EVENT.HIDE_ALL, hide);
    return function () {
      _eventManager.eventManager.off(id, show).off(_constants.EVENT.HIDE_ALL, hide);
    };
    // hide rely on setState(dispatch), which is guaranted to be the same across render
  }, [id, animation, disableBoundariesCheck]);

  // collect menu items for keyboard navigation
  (0, _react.useEffect)(function () {
    !state.visible ? itemTracker.clear() : menuController.init(itemTracker);
  }, [state.visible, menuController, itemTracker]);
  function checkBoundaries(x, y) {
    if (nodeRef.current && !disableBoundariesCheck) {
      var _window = window,
        innerWidth = _window.innerWidth,
        innerHeight = _window.innerHeight;
      var _nodeRef$current = nodeRef.current,
        offsetWidth = _nodeRef$current.offsetWidth,
        offsetHeight = _nodeRef$current.offsetHeight;
      if (x + offsetWidth > innerWidth) x -= x + offsetWidth - innerWidth;
      if (y + offsetHeight > innerHeight) y -= y + offsetHeight - innerHeight;
    }
    return {
      x: x,
      y: y
    };
  }

  // when the menu is transitioning from not visible to visible,
  // the nodeRef is attached to the dom element this let us check the boundaries
  (0, _react.useEffect)(function () {
    // state.visible and state{x,y} are updated together
    if (state.visible) setState(checkBoundaries(state.x, state.y));
  }, [state.visible]);

  // subscribe dom events
  (0, _react.useEffect)(function () {
    function preventDefault(e) {
      if (preventDefaultOnKeydown) e.preventDefault();
    }
    function handleKeyboard(e) {
      switch (e.key) {
        case 'Enter':
        case ' ':
          if (!menuController.openSubmenu()) hide();
          break;
        case 'Escape':
          hide();
          break;
        case 'ArrowUp':
          preventDefault(e);
          menuController.moveUp();
          break;
        case 'ArrowDown':
          preventDefault(e);
          menuController.moveDown();
          break;
        case 'ArrowRight':
          preventDefault(e);
          menuController.openSubmenu();
          break;
        case 'ArrowLeft':
          preventDefault(e);
          menuController.closeSubmenu();
          break;
        default:
          menuController.matchKeys(e);
          break;
      }
    }
    if (state.visible) {
      window.addEventListener('keydown', handleKeyboard);
      for (var _iterator = _createForOfIteratorHelperLoose(_constants.hideOnEvents), _step; !(_step = _iterator()).done;) {
        var ev = _step.value;
        window.addEventListener(ev, hide);
      }
    }
    return function () {
      window.removeEventListener('keydown', handleKeyboard);
      for (var _iterator2 = _createForOfIteratorHelperLoose(_constants.hideOnEvents), _step2; !(_step2 = _iterator2()).done;) {
        var _ev = _step2.value;
        window.removeEventListener(_ev, hide);
      }
    };
  }, [state.visible, menuController, preventDefaultOnKeydown]);
  function show(_ref2) {
    var event = _ref2.event,
      props = _ref2.props,
      position = _ref2.position,
      children = _ref2.children;
    event.stopPropagation();
    var p = position || (0, _utils.getMousePosition)(event);
    // check boundaries when the menu is already visible
    var _checkBoundaries = checkBoundaries(p.x, p.y),
      x = _checkBoundaries.x,
      y = _checkBoundaries.y;
    (0, _reactDom.flushSync)(function () {
      setState({
        visible: true,
        willLeave: false,
        x: x,
        y: y,
        triggerEvent: event,
        propsFromTrigger: props,
        children: children
      });
    });
    window.clearTimeout(visibilityId.current);
    if (!wasVisible.current && (0, _utils.isFn)(onVisibilityChange)) {
      onVisibilityChange(true);
      wasVisible.current = true;
    }
  }
  function hide(e) {
    if (e != null && (
    // Safari trigger a click event when you ctrl + trackpad
    e.button === 2 || e.ctrlKey) &&
    // Firefox trigger a click event when right click occur
    e.type !== 'contextmenu') return;
    animation && ((0, _utils.isStr)(animation) || 'exit' in animation && animation.exit) ? setState(function (state) {
      return {
        willLeave: state.visible
      };
    }) : setState(function (state) {
      return {
        visible: state.visible ? false : state.visible
      };
    });
    visibilityId.current = window.setTimeout(function () {
      (0, _utils.isFn)(onVisibilityChange) && onVisibilityChange(false);
      wasVisible.current = false;
    });
  }
  function handleAnimationEnd() {
    if (state.willLeave && state.visible) {
      (0, _reactDom.flushSync)(function () {
        return setState({
          visible: false,
          willLeave: false
        });
      });
    }
  }
  function computeAnimationClasses() {
    if ((0, _utils.isStr)(animation)) {
      var _cx;
      return (0, _classnames["default"])((_cx = {}, _cx["" + _constants.CssClass.animationWillEnter + animation] = visible && !willLeave, _cx["" + _constants.CssClass.animationWillLeave + animation + " " + _constants.CssClass.animationWillLeave + "'disabled'"] = visible && willLeave, _cx));
    } else if (animation && 'enter' in animation && 'exit' in animation) {
      var _cx2;
      return (0, _classnames["default"])((_cx2 = {}, _cx2["" + _constants.CssClass.animationWillEnter + animation.enter] = animation.enter && visible && !willLeave, _cx2["" + _constants.CssClass.animationWillLeave + animation.exit + " " + _constants.CssClass.animationWillLeave + "'disabled'"] = animation.exit && visible && willLeave, _cx2));
    }
    return null;
  }
  var visible = state.visible,
    triggerEvent = state.triggerEvent,
    propsFromTrigger = state.propsFromTrigger,
    x = state.x,
    y = state.y,
    willLeave = state.willLeave,
    children = state.children;
  var cssClasses = (0, _classnames["default"])(_constants.CssClass.menu, className, (_cx3 = {}, _cx3["" + _constants.CssClass.theme + theme] = theme, _cx3), computeAnimationClasses());

  // TODO: switch to translate instead of top left
  // requires an additional dom element around the menu
  return /*#__PURE__*/_react["default"].createElement(_ItemTrackerProvider.ItemTrackerProvider, {
    value: itemTracker
  }, visible && /*#__PURE__*/_react["default"].createElement("div", (0, _extends2["default"])({}, rest, {
    className: cssClasses,
    onAnimationEnd: handleAnimationEnd,
    style: (0, _extends2["default"])({}, style, {
      left: x,
      top: y,
      opacity: 1
    }),
    ref: nodeRef,
    role: "menu"
  }), (0, _utils.cloneItems)(children, {
    propsFromTrigger: propsFromTrigger,
    triggerEvent: triggerEvent
  })));
};